@@ -32,6 +32,7 @@
 #include "linglong/repo/config.h"
 #include "linglong/runtime/container_builder.h"
 #include "linglong/runtime/run_context.h"
+#include "linglong/utils/bash_quote.h"
 #include "linglong/utils/error/error.h"
 #include "linglong/utils/finally/finally.h"
 #include "linglong/utils/gettext.h"
@@ -42,6 +43,7 @@
 #include "ocppi/runtime/Signal.hpp"
 #include "ocppi/types/ContainerListItem.hpp"
 
+#include <linux/un.h>
 #include <nlohmann/json.hpp>
 
 #include <QCryptographicHash>
@@ -168,7 +170,7 @@ linglong::utils::error::Result<void> ensureDirectory(const std::filesystem::path
     return LINGLONG_OK;
 }
 
-static std::vector<std::string> getAutoModuleList() noexcept
+std::vector<std::string> getAutoModuleList() noexcept
 {
     auto getModuleFromLanguageEnv = [](const std::string &lang) -> std::vector<std::string> {
         if (lang.length() < 2) {
@@ -242,6 +244,82 @@ static std::vector<std::string> getAutoModuleList() noexcept
     return { result.begin(), std::unique(result.begin(), result.end()) };
 }
 
+bool delegateToContainerInit(const std::string &containerID,
+                             std::vector<std::string> commands) noexcept
+{
+    auto containerSocket = ::socket(AF_UNIX, SOCK_SEQPACKET, 0);
+    if (containerSocket == -1) {
+        return false;
+    }
+
+    auto cleanup = linglong::utils::finally::finally([containerSocket] {
+        ::close(containerSocket);
+    });
+
+    struct sockaddr_un addr{};
+    addr.sun_family = AF_UNIX;
+
+    auto bundleDir = linglong::runtime::getBundleDir(containerID);
+    const std::string socketPath = bundleDir / "init/socket";
+
+    std::copy(socketPath.begin(), socketPath.end(), &addr.sun_path[0]);
+    addr.sun_path[socketPath.size() + 1] = 0;
+
+    auto ret = ::connect(containerSocket,
+                         reinterpret_cast<struct sockaddr *>(&addr),
+                         offsetof(sockaddr_un, sun_path) + socketPath.size());
+    if (ret == -1) {
+        return false;
+    }
+
+    std::string bashContent;
+    for (const auto &command : commands) {
+        bashContent.append(linglong::utils::quoteBashArg(command));
+        bashContent.push_back(' ');
+    }
+
+    commands.clear();
+    commands.push_back(bashContent);
+    commands.insert(commands.begin(), "-c");
+    commands.insert(commands.begin(), "--login");
+    commands.insert(commands.begin(), "bash");
+
+    std::string command_data;
+    for (const auto &command : commands) {
+        command_data.append(command);
+        command_data.push_back('\0');
+    }
+    command_data.push_back('\0');
+
+    std::uint64_t rest_len = command_data.size();
+    ret = ::send(containerSocket, &rest_len, sizeof(rest_len), 0);
+    if (ret == -1) {
+        return false;
+    }
+
+    while (rest_len > 0) {
+        auto send_len = ::send(containerSocket, command_data.c_str(), rest_len, 0);
+        if (send_len == -1) {
+            if (errno == EINTR) {
+                continue;
+            }
+
+            break;
+        }
+
+        rest_len -= send_len;
+    }
+
+    if (rest_len != 0) {
+        return false;
+    }
+
+    int result{ -1 };
+    ret = ::recv(containerSocket, &result, sizeof(result), 0);
+    qDebug() << "delegate result:" << result;
+    return result == 0;
+}
+
 } // namespace
 
 namespace linglong::cli {
@@ -529,8 +607,9 @@ int Cli::run([[maybe_unused]] CLI::App *subcommand)
 {
     LINGLONG_TRACE("command run");
 
-    int64_t uid = getuid();
-    int64_t gid = getgid();
+    auto uid = getuid();
+    auto gid = getgid();
+    auto pid = getpid();
 
     // NOTE: ll-box is not support running as root for now.
     if (uid == 0) {
@@ -538,14 +617,14 @@ int Cli::run([[maybe_unused]] CLI::App *subcommand)
         return -1;
     }
 
-    auto userContainerDir = std::filesystem::path{ "/run/linglong" } / std::to_string(::getuid());
+    auto userContainerDir = std::filesystem::path{ "/run/linglong" } / std::to_string(uid);
     if (auto ret = ensureDirectory(userContainerDir); !ret) {
         this->printer.printErr(ret.error());
         return -1;
     }
 
     auto mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
-    auto pidFile = userContainerDir / std::to_string(::getpid());
+    auto pidFile = userContainerDir / std::to_string(pid);
     // placeholder file
     auto fd = ::open(pidFile.c_str(), O_WRONLY | O_CREAT | O_EXCL, mode);
     if (fd == -1) {
@@ -604,11 +683,9 @@ int Cli::run([[maybe_unused]] CLI::App *subcommand)
 
     // this lambda will dump reference of containerID, app, base and runtime to
     // /run/linglong/getuid()/getpid() to store these needed infomation
-    auto dumpContainerInfo = [uid, &runContext, this]() -> bool {
+    auto dumpContainerInfo = [&pidFile, &runContext, this]() -> bool {
         LINGLONG_TRACE("dump info")
         std::error_code ec;
-        auto pidFile = std::filesystem::path{ "/run/linglong" } / std::to_string(uid)
-          / std::to_string(::getpid());
         if (!std::filesystem::exists(pidFile, ec)) {
             if (ec) {
                 qCritical() << "couldn't get status of" << pidFile.c_str() << ":"
@@ -634,8 +711,8 @@ int Cli::run([[maybe_unused]] CLI::App *subcommand)
 
     auto containers = getCurrentContainers().value_or(std::vector<api::types::v1::CliContainer>{});
     for (const auto &container : containers) {
+        qDebug() << "found running container: " << container.package.c_str();
         if (container.package != curAppRef->toString().toStdString()) {
-            qDebug() << "mismatch:" << container.package.c_str() << " -- " << curAppRef->toString();
             continue;
         }
 
@@ -643,47 +720,12 @@ int Cli::run([[maybe_unused]] CLI::App *subcommand)
             return -1;
         }
 
-        QStringList execArgs;
-        std::transform(commands.begin(),
-                       commands.end(),
-                       std::back_inserter(execArgs),
-                       [](const std::string &arg) {
-                           return QString::fromStdString(arg);
-                       });
-
-        // 为避免原始args包含空格，每个arg都使用单引号包裹，并对arg内部的单引号进行转义替换
-        std::for_each(execArgs.begin(), execArgs.end(), [](QString &arg) {
-            arg.replace("'", "'\\''");
-            arg.prepend('\'');
-            arg.push_back('\'');
-        });
-
-        // exec命令使用原始args中的进程替换bash进程
-        execArgs.prepend("exec");
-
-        // 在原始args前面添加bash --login -c，这样可以使用/etc/profile配置的环境变量
-        commands = std::vector<std::string>{ "/bin/bash",
-                                             "--login",
-                                             "-e",
-                                             "-c",
-                                             execArgs.join(' ').toStdString() };
-
-        auto opt = ocppi::runtime::ExecOption{
-            .uid = uid,
-            .gid = gid,
-        };
-
-        auto result = this->ociCLI.exec(container.id,
-                                        commands[0],
-                                        { commands.cbegin() + 1, commands.cend() },
-                                        opt);
-        if (!result) {
-            auto err = LINGLONG_ERRV(result);
-            this->printer.printErr(err);
-            return -1;
+        if (delegateToContainerInit(container.id, commands)) {
+            return 0;
         }
 
-        return 0;
+        // fallback to run
+        break;
     }
 
     auto *homeEnv = ::getenv("HOME");
@@ -698,7 +740,17 @@ int Cli::run([[maybe_unused]] CLI::App *subcommand)
         this->printer.printErr(res.error());
         return -1;
     }
+
+    std::error_code ec;
+    auto socketDir = cfgBuilder.getBundlePath() / "init";
+    std::filesystem::create_directories(socketDir, ec);
+    if (ec) {
+        this->printer.printErr(LINGLONG_ERRV(ec.message().c_str()));
+        return -1;
+    }
+
     cfgBuilder.setAppId(curAppRef->id.toStdString())
+      .setAnnotation(generator::ANNOTATION::LAST_PID, std::to_string(pid))
       .addUIdMapping(uid, uid, 1)
       .addGIdMapping(gid, gid, 1)
       .bindDefault()
@@ -715,7 +767,20 @@ int Cli::run([[maybe_unused]] CLI::App *subcommand)
       .mapPrivate(std::string{ homeEnv } + "/.gnupg", true)
       .bindIPC()
       .forwardDefaultEnv()
-      .enableSelfAdjustingMount();
+      .enableSelfAdjustingMount()
+      .addExtraMount(
+        ocppi::runtime::config::types::Mount{ .destination = "/run/linglong/init",
+                                              .options = std::vector<std::string>{ "bind" },
+                                              .source = socketDir.string(),
+                                              .type = "bind" });
+
+    for (const auto &env : options.envs) {
+        auto split = env.cbegin() + env.find('='); // already checked by CLI
+        cfgBuilder.appendEnv(std::string(env.cbegin(), split),
+                             std::string(split + 1, env.cend()),
+                             true);
+    }
+
 #ifdef LINGLONG_FONT_CACHE_GENERATOR
     cfgBuilder.enableFontCache();
 #endif
@@ -844,6 +909,10 @@ Cli::getCurrentContainers() const noexcept
             continue;
         }
 
+        if (pidFile.file_size(ec) == 0) {
+            continue;
+        }
+
         auto info = linglong::utils::serialize::LoadJSONFile<
           linglong::api::types::v1::ContainerProcessStateInfo>(file);
         if (!info) {
@@ -2737,13 +2806,18 @@ int Cli::dir([[maybe_unused]] CLI::App *subcommand)
         return -1;
     }
 
-    auto layerItem = this->repository.getLayerItem(*ref);
-    if (!layerItem) {
-        this->printer.printErr(layerItem.error());
+    std::string module = "binary";
+    if (!options.module.empty()) {
+        module = options.module;
+    }
+
+    auto layerDir = this->repository.getLayerDir(*ref, module);
+    if (!layerDir) {
+        this->printer.printErr(layerDir.error());
         return -1;
     }
 
-    std::cout << layerItem->commit << std::endl;
+    std::cout << layerDir->absolutePath().toStdString() << std::endl;
     return 0;
 }
 
